import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'variables.dart';
import 'function.dart';
import 'dart:convert';

void getDevices(){

  var devicesUUIDs = storage.controller.get(':Devices').getChildNodesCsv().split(',');

  for(var UUID in devicesUUIDs){

    var deviceNodes = storage.controller.get(':Devices:$UUID').getChildren().entries.iterator;

    try {
        if (deviceNodes.current.value.getValue('owner')!.getValue ('en')
              ! == storage.userUUID){


              var deviceScoreNode = storage.controller.get(':Devices:$UUID:' + storage.GEIGERindicatorUUID + ':data:GeigerScoreDevice');

              var GeigerDevScore = deviceScoreNode.getValue('GEIGER_score')!.getValue('en')!;

              var GeigerDevNumMetrics = deviceScoreNode.getValue('number_metrics')!.getValue('en')!;

              Node scoreNum = NodeImpl('');
              scoreNum.addValue(NodeValueImpl('score', GeigerDevScore));
              scoreNum.addValue(NodeValueImpl('nums', GeigerDevNumMetrics));
              coreValues.aggregate.add(scoreNum);

        }
      }catch (e){
          continue;
    }


  }

}

void getEmployees(){


  var userNode = storage.controller.get(':Enterprise:users:'+storage.userUUID!);

  var flagmainUser = userNode.getValue('mainUser')!.getValue('en');
  var employeeNode = userNode.getValue('supervising')!.getValue('en');

  coreValues.flagMainUser = flagmainUser == '1' ? true : false;

  if(employeeNode != null){

    var employees = employeeNode.split(',');
    var userAggregateScoreNode;
      for (var empUUID in employees) {


          var userAggregateScoreNodeName = ':Users:' + empUUID + ':' + storage.GEIGERindicatorUUID + ':data:GeigerScoreAggregate';

          try{
             userAggregateScoreNode = storage.controller.get(userAggregateScoreNodeName);
          }catch(e){continue;}

          var GeigerAggScore = userAggregateScoreNode.getValue('GEIGER_score')!.getValue('en')!;
          var GeigerAggNumMetrics = userAggregateScoreNode.getValue('number_metrics')!.getValue('en')!;

          Node scoreNums = NodeImpl('');
          scoreNums.addValue(NodeValueImpl('score', GeigerAggScore));
          scoreNums.addValue(NodeValueImpl('nums', GeigerAggNumMetrics));
          coreValues.aggregate.add(scoreNums);

      }

  }

}


String getEnterpriseUUID(){

  var enterpriseNode = storage.controller.get(':Enterprise');

  var enterpriseNodeIterator = enterpriseNode.getChildren().entries.iterator;
  var profile;

  while(enterpriseNodeIterator.moveNext()) {

      try {
            var sector = enterpriseNodeIterator.current.value.getValue('sector')!.getValue('en')!;
            var location = enterpriseNodeIterator.current.value.getValue('location')!.getValue('en')!;
             profile = enterpriseNodeIterator.current.value.getValue('associatedProfiles')!.getValue('en')!;

            coreValues.enterpriseNode['sector'] = sector;
            coreValues.enterpriseNode['location'] = location;
            coreValues.enterpriseNode['associatedProfiles'] = profile;
      }catch(e){

          //print(e);
          continue;
      }
  }

  return profile;
}


void getRisk(){


  var profilesNode = storage.controller.get(':Global:profiles');

  var profileNodeIterator = profilesNode.getChildren().entries.iterator;

  while(profileNodeIterator.moveNext()){

  var profileName = profileNodeIterator.current.value.getValue('name')!.getValue('en')!.toLowerCase();

  for(String threatUUID in coreValues.uuidThreats){

        switch(profileName){

          case types.digitallyBased:
                coreValues.based[threatUUID] = profileNodeIterator.current.value.getValue(threatUUID)!.getValue('en')!;
            break;

          case types.digitalEnabler:
                coreValues.enabler[threatUUID] = profileNodeIterator.current.value.getValue(threatUUID)!.getValue('en')!;
            break;

          case types.digitallyDependent:
                coreValues.dependent[threatUUID] = profileNodeIterator.current.value.getValue(threatUUID)!.getValue('en')!;
            break;
        }

  }

}

}


void getRecommendation(){

  var recommendationNode = storage.controller.get(':Global:Recommendations');

   var recoNodeIterator = recommendationNode.getChildren().entries.iterator;

   while(recoNodeIterator.moveNext()) {
     var RecommendationType = recoNodeIterator.current.value.getValue(
         'RecommendationType')!.getValue('en')!;


        var lists = recoNodeIterator.current.value.getValue('relatedThreatsWeights')!.getValue('en')!.split(';'); //List<String>


        for(var node in lists){

          var relatedThreatsWeights = node.split(',');

          Node impactNormal = NodeImpl('');
          impactNormal.addValue(NodeValueImpl('impact', relatedThreatsWeights[1]));
          impactNormal.addValue(NodeValueImpl('recuuid', recoNodeIterator.current.key));
          impactNormal.addValue(NodeValueImpl('short', recoNodeIterator.current.value.getValue('short')!.getValue('en')!));

          //RecommendationType == types.Devices ? coreValues.deviceRecommendation[relatedThreatsWeights[0]]!.add(impactNormal) : coreValues.userRecommendation[relatedThreatsWeights[0]]!.add(impactNormal);


            if(RecommendationType == types.Devices.toLowerCase()){

                try{
                  coreValues.deviceRecommendation[relatedThreatsWeights[0]]!.add(impactNormal);
                }catch(e){
                  var temp = <Node>[];
                  temp.add(impactNormal);
                  coreValues.deviceRecommendation[relatedThreatsWeights[0]] = temp;
                }

            }else{

                try{
                  coreValues.userRecommendation[relatedThreatsWeights[0]]!.add(impactNormal);
                }catch(e){
                  var temp = <Node>[];
                  temp.add(impactNormal);
                  coreValues.userRecommendation[relatedThreatsWeights[0]] = temp;
                }
            }

        }
   }


}


void getMetrics(var type, var UUID) {


  var MetricNode;


      for(var pluginsUUID in  storage.pluginUUIDs) {

       //  print(':$type:' + UUID+':'+pluginsUUID+':data:metrics');
         try {
            MetricNode = storage.controller.get(':$type:' + UUID + ':' + pluginsUUID + ':data:metrics');
         }
         catch(e) { continue;}

         var metricNodeIterator = MetricNode.getChildren().entries.iterator;

         while(metricNodeIterator.moveNext()){

           var normalizationValue = normalization(metricNodeIterator);


           var threatsImpacts = metricNodeIterator.current.value.getValue('threatsImpact')!.getValue('en')!.split(';');


           for(var list in threatsImpacts){

                var threatsImpact = list.split(',');


              //  print(metricNodeIterator.current.value.getValue('flag')!.getValue('en'));

                if(metricNodeIterator.current.value.getValue('flag')!.getValue('en') == '1'){


                  Node impactNormal = NodeImpl('');
                  impactNormal.addValue(NodeValueImpl('impact', threatsImpact[1]));
                  impactNormal.addValue(NodeValueImpl('normal', normalizationValue.toString()));


                  //type == types.Devices ? coreValues.devicePositive[threatsImpact[0]]!.add(impactNormal) : coreValues.userPositive[threatsImpact[0]]!.add(impactNormal);

                  if(type == types.Devices){

                      try{
                          coreValues.devicePositive[threatsImpact[0]]!.add(impactNormal);
                      }catch(e){
                          var temp = <Node>[];
                          temp.add(impactNormal);
                          coreValues.devicePositive[threatsImpact[0]] = temp;
                      }

                  }else{

                      try{
                          coreValues.userPositive[threatsImpact[0]]!.add(impactNormal);
                      }catch(e){
                      var temp = <Node>[];
                      temp.add(impactNormal);
                      coreValues.userPositive[threatsImpact[0]] = temp;
                      }
                  }


                }else{

                  Node impactNormal = NodeImpl('');
                  impactNormal.addValue(NodeValueImpl('impact', threatsImpact[1]));
                  impactNormal.addValue(NodeValueImpl('normal', normalizationValue.toString()));
               //   type == types.Devices ? coreValues.deviceNegative[threatsImpact[0]]!.add(impactNormal) : coreValues.userNegative[threatsImpact[0]]!.add(impactNormal);

                  if(type == types.Devices){

                      try{
                        coreValues.deviceNegative[threatsImpact[0]]!.add(impactNormal);
                      }catch(e){
                        var temp = <Node>[];
                        temp.add(impactNormal);
                        coreValues.deviceNegative[threatsImpact[0]] = temp;
                      }

                  }else{

                      try{
                        coreValues.userNegative[threatsImpact[0]]!.add(impactNormal);
                      }catch(e){
                        var temp = <Node>[];
                        temp.add(impactNormal);
                        coreValues.userNegative[threatsImpact[0]] = temp;
                      }
                  }

                }

            }

         }

      }

}


void getExistScoreNodes(var type, var path){


    try {

      var node =  storage.controller.get(':$type:' + storage.userUUID! + ':' + storage.GEIGERindicatorUUID + ':data:$path');

      if(path == types.GeigerScoreUser || path == types.GeigerScoreDevice){

          coreValues.existReco = node.getValue('implementedRecommendations')!.getValue('en')!.split(',');

      }


    } catch(e){
          //Creation
          print('\n');
          print('Node : $type is not found');


          if(path == types.GeigerScoreUser || path == types.GeigerScoreDevice || path == types.GeigerScoreAggregate){
              createScoreNode(type, path); //initailzation
          } else if(path == types.GeigerScoreMSE){

              createScoreMSENode(type, path);

          } else if(path == types.Recommendations){
              createRecommendationNode(type);

          }
    }


}



void createRecommendationNode(var type){

  var UUID;

  type == types.Users  ? UUID = storage.userUUID : UUID = storage.deviceUUID;

  Node RecommendationNode =  NodeImpl('recommendations', ':$type:' + UUID + ':' + storage.GEIGERindicatorUUID + ':data');

  storage.controller.add(RecommendationNode);

}

void updateRecommendationNode(var type){

  var UUID;

  type == types.Users  ? UUID = storage.userUUID : UUID = storage.deviceUUID;

  var RecommendationNode = storage.controller.get(':$type:' + UUID + ':' + storage.GEIGERindicatorUUID + ':data:recommendations');

  var threatRecoList = crtRecommendation(type);

  for (NodeValue tr in threatRecoList ){
    RecommendationNode.addValue(tr);
  }
  
  //print(RecommendationNode);
  storage.controller.update(RecommendationNode);
}


void createScoreMSENode(var type, var path){

  storage.controller.add(NodeImpl(storage.GEIGERindicatorUUID, ':$type'));
  storage.controller.add(NodeImpl('data', ':$type:' + storage.GEIGERindicatorUUID));

  Node ScoreMSENode =  NodeImpl(path, ':$type:' + storage.GEIGERindicatorUUID + ':data');

  //create node keys: UserScore, ImplementRecommendations
  NodeValue GEIGER_score =  NodeValueImpl('GEIGER_score', '0'); //add node name (name of threat)
  ScoreMSENode.addValue(GEIGER_score);

  NodeValue location =  NodeValueImpl('location', '');
  ScoreMSENode.addValue(location);

  NodeValue sector =  NodeValueImpl('sector', '0');
  ScoreMSENode.addValue(sector);

  NodeValue associatedProfiles =  NodeValueImpl('associatedProfiles', '0');
  ScoreMSENode.addValue(associatedProfiles);

  GEIGER_score.setDescription('GEIGER user score');


  storage.controller.add(ScoreMSENode);
}

void updateScoreMSENode(){

  if(coreValues.flagMainUser) {
    var ScoreMSENode = storage.controller.get(
        ':Enterprise:' + storage.GEIGERindicatorUUID +
            ':data:GeigerScoreMSE');


    //create node keys: UserScore, ImplementRecommendations
    NodeValue GEIGER_score = NodeValueImpl(
        'GEIGER_score', coreValues.gagg.toString()); //add node name (name of threat)
    ScoreMSENode.updateValue(GEIGER_score);

    NodeValue location = NodeValueImpl(
        'location', coreValues.enterpriseNode['location']);
    ScoreMSENode.updateValue(location);

    NodeValue sector = NodeValueImpl(
        'sector', coreValues.enterpriseNode['sector']);
    ScoreMSENode.updateValue(sector);

    NodeValue associatedProfiles = NodeValueImpl(
        'associatedProfiles', coreValues.enterpriseNode['associatedProfiles']);
    ScoreMSENode.updateValue(associatedProfiles);


    storage.controller.update(ScoreMSENode);
   // print(ScoreMSENode);
  }
}


void createScoreNode(var type, var path){


  Node GeigerIndicatorUserScoreNode =  NodeImpl(path, ':$type:' + storage.userUUID! + ':' + storage.GEIGERindicatorUUID + ':data');

  //create node keys: UserScore, ImplementRecommendations
  NodeValue GEIGER_score =  NodeValueImpl('GEIGER_score', '0'); //add node name (name of threat)
  GeigerIndicatorUserScoreNode.addValue(GEIGER_score);

  NodeValue GEIGERThreatscores =  NodeValueImpl('threats_score', '');
  GeigerIndicatorUserScoreNode.addValue(GEIGERThreatscores);

  NodeValue GEIGERnumMetrics =  NodeValueImpl('number_metrics', '0');
  GeigerIndicatorUserScoreNode.addValue(GEIGERnumMetrics);

  NodeValue geigerJsonObject =  NodeValueImpl('geigerJsonObj', '');
  GeigerIndicatorUserScoreNode.addValue(geigerJsonObject);

  GEIGER_score.setDescription('GEIGER user score');
  GEIGERThreatscores.setDescription('GEIGER threat-specific user score');
  GEIGERnumMetrics.setDescription('Number of metrics used in calculation of user score');

  if(path != types.GeigerScoreAggregate) {
    storage.controller.add(NodeImpl(storage.userUUID!, ':$type'));
    storage.controller.add(NodeImpl(storage.GEIGERindicatorUUID, ':$type:' + storage.userUUID!));
    storage.controller.add(NodeImpl('data', ':$type:' + storage.userUUID! + ':' + storage.GEIGERindicatorUUID));

    NodeValue implementedRecommendations = NodeValueImpl('implementedRecommendations', ''); //add node name (name of threat)
    GeigerIndicatorUserScoreNode.addValue(implementedRecommendations);
  }

  storage.controller.add(GeigerIndicatorUserScoreNode);
}


void updateScoreNode(var type, var path){


  var scoreNode = storage.controller.get(':$type:' + storage.userUUID! + ':' + storage.GEIGERindicatorUUID + ':data:$path');

  crtThreatScore(type);

  NodeValue GEIGERThreatscores =  NodeValueImpl('threats_score', jsonEncode(jsonObject.threatScoreValues));
  scoreNode.updateValue(GEIGERThreatscores);

  var GEIGER_score;
  type == types.GeigerScoreAggregate ? GEIGER_score = coreValues.gagg : GEIGER_score = crtTotalThreatScore(type);

  NodeValue GEIGERscore =  NodeValueImpl('GEIGER_score', GEIGER_score);
  scoreNode.updateValue(GEIGERscore);


  jsonObject.geigerNode['threatScores'] = jsonObject.threatScoreValues;
  jsonObject.geigerNode['geigerScore'] = GEIGER_score;


  var len;
  type == types.Devices? len = coreValues.devicePositive.length + coreValues.deviceNegative.length : len = coreValues.userPositive.length + coreValues.userNegative.length;

  var number_metrics;
  type == types.GeigerScoreAggregate ? number_metrics = coreValues.nagg : number_metrics = len;

  NodeValue GEIGERnumMetrics =  NodeValueImpl('number_metrics', number_metrics.toString() );
  scoreNode.updateValue(GEIGERnumMetrics);


  if(path != types.GeigerScoreAggregate) {

    NodeValue implementedRecommendations = NodeValueImpl('implementedRecommendations', ''); //?????
    scoreNode.updateValue(implementedRecommendations);

    var objectName;
    type == types.Devices ? objectName= 'geigerScoreDevice' : objectName = 'geigerScoreUsers';
    jsonObject.geigerNodeKey['$objectName'] = jsonObject.geigerNode;

    type == types.Devices ? jsonObject.geigerScoreDevice= jsonObject.geigerNode : jsonObject.geigerScoreUser = jsonObject.geigerNode;

  } else {
    jsonObject.geigerNodeKey['geigerScoreAggregate'] = jsonObject.geigerNode;
    jsonObject.geigerScoreAggregate = jsonObject.geigerNode;
  }


  NodeValue geigerJsonObject =  NodeValueImpl('geigerJsonObj', jsonEncode(jsonObject.geigerNodeKey));
  scoreNode.updateValue(geigerJsonObject);

  storage.controller.update(scoreNode);
 // print(scoreNode);



}



