import '../variables.dart';
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:csv/csv.dart';
import 'dart:io';
import 'dart:convert';
import '../function.dart';

void createLocal(){ //Who?

  Node localNode = NodeImpl(':Local');

  NodeValue user_uuid = NodeValueImpl('currentUser', '328161f6-89bd-49f6-user-5a375ff56ana');
  localNode.addValue(user_uuid);

  NodeValue device_uuid = NodeValueImpl('currentDevice', '328161f6-89bd-49f6-dev1-5a375ff56ana');
  localNode.addValue(device_uuid);

  storage.controller.update(localNode);

  //print(storage.controller.get(":Local"));

}


Future createThreats() async {


    Node threats = NodeImpl(':Global:threats');
    storage.controller.add(threats);

    final input =  File('lib/src/dummyAPI/dummydata/threat.csv').openRead();
    final fields = await input.transform(utf8.decoder).transform(CsvToListConverter()).toList();

    for ( var str in fields[0].toString().replaceAll('[', '').split('\n') ){

      try {
        var values = str.split(',');
        Node temp = NodeImpl(values[0], ':Global:threats');
        temp.addValue(NodeValueImpl('name', values[1]));
        temp.addValue(NodeValueImpl('GEIGER_threat', values[2]));
        threats.addChild(temp);
      }
      // ignore: empty_catches
      catch(e){

      }
    }

  storage.controller.update(threats);

  //print(storage.controller.get(':Global:threats').getChildNodesCsv());

}

Future createPlugin() async{

  //:Local:plugin
  Node plugins = NodeImpl(':Local:plugin');
  storage.controller.add(plugins);

  final input =  File('lib/src/dummyAPI/dummydata/local.csv').openRead();
  final fields = await input.transform(utf8.decoder).transform(CsvToListConverter()).toList();

  for ( var str in fields[0].toString().replaceAll('[', '').split('\n') ){

    try {
      var values = str.split(',');
      Node temp = NodeImpl(values[0], ':Local:plugin');
      temp.addValue(NodeValueImpl('name', values[1]));
      plugins.addChild(temp);
    }
    // ignore: empty_catches
    catch(e){

    }
  }

  storage.controller.update(plugins);

  //print(storage.controller.get(':Local:plugin').getChildNodesCsv());
}


Future createProfiles() async{

  //":Global:profiles"
  Node profiles = NodeImpl(':Global:profiles');
  storage.controller.add(profiles);

  Map x = json.decode(await File('lib/src/dummyAPI/dummydata/profiles.json').readAsString());

  var t =  x.entries.iterator;

  while(t.moveNext()){

     var profile = t.current.value;
     Node temp = NodeImpl(profile['UUID'], ':Global:profiles');
     temp.addValue(NodeValueImpl('name', t.current.key));

     Map threats = t.current.value['threats'];

     threats.forEach((key, value) {
       temp.addValue(NodeValueImpl(key, value));
     });

     profiles.addChild(temp);
  }

  storage.controller.update(profiles);

  //print(storage.controller.get(':Global:profiles').getChildren());

}

Future createRecommendations() async{


  Node recommendations = NodeImpl(':Global:Recommendations');
  storage.controller.add(recommendations);

  Map x = json.decode(await File('lib/src/dummyAPI/dummydata/recommendations.json').readAsString());

  var t =  x.entries.iterator;

  while(t.moveNext()){

    var recommendation = t.current.value;
    Node temp = NodeImpl(recommendation['UUID'], ':Global:Recommendations');
    temp.addValue(NodeValueImpl('short', recommendation['short']));
    temp.addValue(NodeValueImpl('relatedThreatsWeights', recommendation['relatedThreatsWeights']));
    temp.addValue(NodeValueImpl('RecommendationType', recommendation['RecommendationType']));
    storage.controller.add(temp);
    recommendations.addChild(temp);
  }



  storage.controller.update(recommendations);

//   print(storage.controller.get(':Global:Recommendations').getChildren());
}

Future createEnterprise() async{

  //:Enterprise
//  Node enterprise = NodeImpl(':Enterprise');
//  storage.controller.add(enterprise);

  final input =  File('lib/src/dummyAPI/dummydata/Enterprise.csv').openRead();
  final fields = await input.transform(utf8.decoder).transform(CsvToListConverter()).toList();

  for ( var str in fields[0].toString().replaceAll('[', '').split('\n') ){

    try {
      var values = str.split(',');
      Node temp = NodeImpl(values[0], ':Enterprise');
      temp.addValue(NodeValueImpl('name', values[1]));
      temp.addValue(NodeValueImpl('location', values[2]));
      temp.addValue(NodeValueImpl('sector', values[3]));
      temp.addValue(NodeValueImpl('associatedProfiles', values[4]));
      storage.controller.add(temp);
    }
    // ignore: empty_catches
    catch(e){

    }
  }


 // print(storage.controller.get(':Enterprise').getChildren());

}


Future createEnterpriseDevices() async{

  //:Enterprise
//  Node enterprise = NodeImpl(':Enterprise');
//  storage.controller.add(enterprise);

  final input =  File('lib/src/dummyAPI/dummydata/enterpriseDevices.csv').openRead();
  final fields = await input.transform(utf8.decoder).transform(CsvToListConverter()).toList();


  for ( var str in fields[0].toString().replaceAll('[', '').split('\n') ){

    try {
      var values = str.split(',');
      Node temp = NodeImpl(values[0], ':Devices');
      temp.addValue(NodeValueImpl('name', values[1]));
      temp.addValue(NodeValueImpl('type', values[2]));
      temp.addValue(NodeValueImpl('owner', values[3]));
      storage.controller.add(temp);
    }
    // ignore: empty_catches
    catch(e){

    }
  }


 //print(storage.controller.get(':Devices').getChildren());

}


Future createEnterpriseUsers() async {

  //":Global:profiles"
  Node enterpriseUsers = NodeImpl(':Enterprise:users');
  storage.controller.add(enterpriseUsers);

  Map x = json.decode(await File('lib/src/dummyAPI/dummydata/enterpriseUsers.json').readAsString());

  var t =  x.entries.iterator;

  while(t.moveNext()){

    var nodeArrays = t.current.value;

    for(var node in nodeArrays){

      Node temp = NodeImpl(node['UUID'], ':Enterprise:users');
      temp.addValue(NodeValueImpl('name',node['name']));
      temp.addValue(NodeValueImpl('firstName',node['firstName']));
      temp.addValue(NodeValueImpl('UUIDs',node['UUIDs']));
      temp.addValue(NodeValueImpl('mainUser',node['mainUser']));
      temp.addValue(NodeValueImpl('supervising',node['supervising']));

      enterpriseUsers.addChild(temp);

    }


  }

  storage.controller.update(enterpriseUsers);


}


Future createUsers() async{



  final input =  File('lib/src/dummyAPI/dummydata/Users.csv').openRead();
  final fields = await input.transform(utf8.decoder).transform(CsvToListConverter()).toList();

  for ( var str in fields[0].toString().replaceAll('[', '').split('\n') ){

    try {
      var values = str.split(',');

      storage.controller.add(NodeImpl(values[0], ':Users'));
      storage.controller.add(NodeImpl(storage.GEIGERindicatorUUID, ':Users:' + values[0]));
      storage.controller.add(NodeImpl('data', ':Users:' + values[0] + ':' + storage.GEIGERindicatorUUID));

      Node temp = NodeImpl('GeigerScoreUser', ':Users:' + values[0] + ':' + storage.GEIGERindicatorUUID + ':data');
      temp.addValue(NodeValueImpl('GEIGER_score', values[1]));
      temp.addValue(NodeValueImpl('implementedRecommendations', values[2]));
      temp.addValue(NodeValueImpl('threats_score', values[3]));
      temp.addValue(NodeValueImpl('number_metrics', values[4]));
      temp.addValue(NodeValueImpl('geigerJsonObj', ''));
      storage.controller.add(temp);
    }
    // ignore: empty_catches
    catch(e){

    }

  }

  //print(storage.controller.get(':Users').getChildren());
  //print(storage.controller.get(':Users:' + storage.userUUID! + ':' + storage.GEIGERindicatorUUID + ':data:GeigerScoreUser'));

}

Future createDevices() async{



  final input =  File('lib/src/dummyAPI/dummydata/Devices.csv').openRead();
  final fields = await input.transform(utf8.decoder).transform(CsvToListConverter()).toList();

  for ( var str in fields[0].toString().replaceAll('[', '').split('\n') ){
    var values = str.split(',');

    try {


      storage.controller.add(NodeImpl(values[0], ':Devices'));
      storage.controller.add(NodeImpl(storage.GEIGERindicatorUUID, ':Devices:' + values[0]));
      storage.controller.add(NodeImpl('data', ':Devices:' + values[0] + ':' + storage.GEIGERindicatorUUID));

      Node temp = NodeImpl('GeigerScoreDevice', ':Devices:' + values[0] + ':' + storage.GEIGERindicatorUUID + ':data');
      temp.addValue(NodeValueImpl('GEIGER_score', values[1]));
      temp.addValue(NodeValueImpl('implementedRecommendations', values[2]));
      temp.addValue(NodeValueImpl('threats_score', values[3]));
      temp.addValue(NodeValueImpl('number_metrics', values[4]));
      temp.addValue(NodeValueImpl('geigerJsonObj', ''));
      storage.controller.add(temp);
    }
    catch(e){

     // print(e);

      storage.controller.add(NodeImpl(storage.GEIGERindicatorUUID, ':Devices:' + values[0]));
      storage.controller.add(NodeImpl('data', ':Devices:' + values[0] + ':' + storage.GEIGERindicatorUUID));

      Node temp = NodeImpl('GeigerScoreDevice', ':Devices:' + values[0] + ':' + storage.GEIGERindicatorUUID + ':data');
      temp.addValue(NodeValueImpl('GEIGER_score', values[1]));
      temp.addValue(NodeValueImpl('implementedRecommendations', values[2]));
      temp.addValue(NodeValueImpl('threats_score', values[3]));
      temp.addValue(NodeValueImpl('number_metrics', values[4]));
      temp.addValue(NodeValueImpl('geigerJsonObj', ''));
      storage.controller.add(temp);

    }

  }
  //print(storage.controller.get(':Devices').getChildren());
//  print(storage.controller.get(':Devices:' + storage.deviceUUID! + ':' + storage.GEIGERindicatorUUID + ':data:GeigerScoreDevice'));

}


