import 'variables.dart';
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'api.dart';
import 'dummyAPI/creation.dart';
import 'dummyAPI/metricCreation.dart';

Future init() async{

  createLocal();
  await createThreats();
  await createPlugin();
  await createProfiles();
  await createRecommendations();
  await createEnterprise();
  await createEnterpriseDevices();
  await createEnterpriseUsers();
  await createUsers();
  await createDevices();

  await createMetricData();
}

void checkExistNodes() {

  generateThreats();
  //generateCoreValues();

  getExistScoreNodes(types.Users,types.GeigerScoreUser);
  getExistScoreNodes(types.Devices,types.GeigerScoreDevice);

  getExistScoreNodes(types.Users,types.Recommendations);
  getExistScoreNodes(types.Devices,types.Recommendations);

  getExistScoreNodes(types.Users,types.GeigerScoreAggregate);
  getExistScoreNodes(types.Enterprise,types.GeigerScoreMSE);
}

void generateCoreValues(){


   for(String uuidThreat in coreValues.uuidThreats){
      var temp = <Node>[];
      coreValues.userPositive[uuidThreat] = temp;
      temp = <Node>[];
      coreValues.userNegative[uuidThreat] = temp;
      temp = <Node>[];
      coreValues.userRecommendation[uuidThreat] = temp;
      temp = <Node>[];
      coreValues.devicePositive[uuidThreat] = temp;
      temp = <Node>[];
      coreValues.deviceNegative[uuidThreat] = temp;
      temp = <Node>[];
      coreValues.deviceRecommendation[uuidThreat] = temp;
   }
}


void generateThreats(){

  try{

    var globalThreats = storage.controller.get(':Global:threats');
    var uuidThreats = globalThreats.getChildNodesCsv().split(',');
    var threatsIterator = globalThreats.getChildren().entries.iterator;
    var uuidIterator = uuidThreats.iterator;
    coreValues.uuidThreats = uuidThreats;

    while(threatsIterator.moveNext()){
      uuidIterator.moveNext();
      var name = threatsIterator.current.value.getValue('name')!.getValue('en');
      coreValues.uuid_name[uuidIterator.current] = name;
      coreValues.name_uuid[name] = uuidIterator.current;
  }


  } on StorageException {

      print ("':Global:Threats' doesn't exist in storage'");
  }


}



double normalization(Iterator<MapEntry<String, Node>> nodes){

  var minValue = double.parse(nodes.current.value.getValue('minValue')!.getValue('en').toString());
  var value = double.parse(nodes.current.value.getValue('GEIGERvalue')!.getValue('en').toString());

  return (value - minValue) / (double.parse(nodes.current.value.getValue('maxValue')!.getValue('en').toString()) - minValue);
}



String crtThreatScore(var type) {


  var positiveMetrics = 0.0;
  var negativeMetrics = 0.0;
  var implementedRecommendations =0.0;
  var Gspt =0.0;
  var treats_scores ='';
  var threatScoreValues = <dynamic>[];

      for (String threatUUID in coreValues.uuidThreats) {

            if (type == types.Devices){

              positiveMetrics = pnMetrics(coreValues.devicePositive, threatUUID);
              negativeMetrics = pnMetrics(coreValues.deviceNegative, threatUUID);

              implementedRecommendations = recoMetrics(coreValues.deviceRecommendation, threatUUID);

            }else if(type == types.Users){

              positiveMetrics = pnMetrics(coreValues.userPositive, threatUUID);
              negativeMetrics = pnMetrics(coreValues.userNegative, threatUUID);
              implementedRecommendations = recoMetrics(coreValues.userRecommendation, threatUUID);

            }

            Gspt = 50 + (50 * negativeMetrics) - (25 * (positiveMetrics + implementedRecommendations));

            var threatValues = <dynamic,dynamic>{};

            threatValues['threatId'] = threatUUID;
            threatValues['name'] = coreValues.uuid_name[threatUUID].toString().trim();

            var threat = <dynamic,dynamic>{};

            threat['threat'] = threatValues;
            threat['score'] = Gspt;

            threatScoreValues.add(threat);

            treats_scores += '$threatUUID,$Gspt;';

            type == types.Devices ? coreValues.deviceThreatScore[threatUUID] = Gspt : coreValues.userThreatScore[threatUUID] = Gspt;
      }

      //print(treats_scores);

     // jsonObject.threatScores['threatScores'] = threatScoreValues;
      jsonObject.threatScoreValues = threatScoreValues;

      return treats_scores;
}


String crtTotalThreatScore(var type) {
  var sumNumerator = 0.0;
  var sumDenominator = 0.0;
  var threatScore;

  type == types.Devices ? threatScore = coreValues.deviceThreatScore : threatScore = coreValues.userThreatScore;

  for (String threatUUID in coreValues.uuidThreats ) { //here loop threat by threat
      var Rpt = riskMetrics(threatUUID);
      sumNumerator += threatScore[threatUUID] * Rpt;
      sumDenominator += Rpt;
  }

  var GspUser = sumNumerator / sumDenominator;

  Node scoreNums = NodeImpl('');
  scoreNums.addValue(NodeValueImpl('score', GspUser.toString()));
  scoreNums.addValue(NodeValueImpl('nums', (coreValues.userPositive.length + coreValues.userNegative.length).toString()));
  coreValues.aggregate.add(scoreNums);


  return GspUser.toString();
}

double riskMetrics(var threatUUID){

  var uuid = getEnterpriseUUID().trim();
  var result = 0.0;

  switch(uuid){

    case types.digitallyBased:
        result = coreValues.based[threatUUID];
      break;

    case types.digitalEnabler:
        result = coreValues.enabler[threatUUID];
      break;

    case types.digitallyDependent:
        result = double.parse(coreValues.dependent[threatUUID]);
      break;
  }

  return result;
}


double pnMetrics(var node, var threatUUID){

  var impactV = 0.0;
  var numerator = 0.0;
  var denominator = 0.0;
  var normalization;
  var metric;
  try{
     metric = node[threatUUID]!.iterator;
  }catch(e){
      return 0;
  }

    while(metric.moveNext()){

          var impact = metric.current.getValue('impact')!.getValue('en')!;
    //      try{
            normalization = metric.current.getValue('normal')!.getValue('en')!;
//          }catch(e){
  //            continue;
//          }

          impactV = coreValues.impactValues[impact.toLowerCase()]!;

          numerator += impactV * double.parse(normalization); //numerator= impact*value
          denominator += impactV; //denom=impact
    }

  return numerator / denominator;
}

double recoMetrics(var node, var threatUUID){

  var impactV = 0.0;
  var numerator = 0.0;
  var denominator = 0.0;

  var metric;
  try{
    metric = node[threatUUID]!.iterator;
  }catch(e){
    return 0;
  }
    while(metric.moveNext()){

        var impact = metric.current.getValue('impact')!.getValue()!;

        impactV = coreValues.impactValues[impact.toLowerCase()]!;

        coreValues.existReco.contains(threatUUID) ? numerator += impactV : denominator += impactV;

    }

  return numerator / denominator;

}


List<dynamic> crtRecommendation(var type){

  var node;
  type == types.Devices ? node = coreValues.deviceRecommendation : node = coreValues.userRecommendation;
  var threatRecoList = [];
  var recUUID;
  var recoObjVList = [];
 // var recoObjList = [];


  for (String threatUUID in coreValues.uuidThreats) {
      var recoLists ='';
      var reco;

      try{
        reco = node[threatUUID]!.iterator;

      }catch(e){
        continue;
      }

      while(reco.moveNext()){

          var impact = reco.current.getValue('impact')!.getValue()!;

          try{
            recUUID = reco.current.getValue('recuuid')!.getValue()!;
          }catch(e){
          continue;
          }
          var impactV = coreValues.impactValues[impact.toLowerCase()]!;

          recoLists += '$recUUID,$impactV;';

          //type =="Devices" ? coreValues.deviceThreatScore[threatUUID] = Gspt : coreValues.userThreatScore[threatUUID] = Gspt;

          var recoVal = {};
          recoVal['weight'] = impactV;
          recoVal['id'] = recUUID;
          recoVal['type'] = type.toString().toLowerCase();
          recoVal['short'] = reco.current.getValue('short')!.getValue()!;

          recoObjVList.add(recoVal);

      }
      threatRecoList.add(NodeValueImpl(threatUUID, recoLists));

      var threatVal = {};
      threatVal['threatId'] = threatUUID;
      threatVal['name'] = coreValues.uuid_name[threatUUID].toString().trim();

      var recoObj = {};
      recoObj['threat'] = threatVal;
      recoObj['recommendations'] = recoObjVList;

      var recoObjKey = {};
      recoObjKey['recommendations'] = recoObj;

      type.toString().toLowerCase() ==types.Devices.toLowerCase() ?  jsonObject.deviceRecoObjList.add(recoObjKey) :  jsonObject.userRecoObjList.add(recoObjKey);
  }

  return threatRecoList;
}



void crtAggregateScore(){

  var sumNumerator = 0.0;
  var sumDenominator = 0.0;


  for (Node scoreNums in coreValues.aggregate) {

    var Gsp = double.parse(scoreNums.getValue('score')!.getValue('en')!);
    var Ns = double.parse(scoreNums.getValue('nums')!.getValue('en')!);
    sumNumerator += (Gsp * Ns);
    sumDenominator += Ns;
  }
  coreValues.gagg = sumNumerator / sumDenominator;
  coreValues.nagg = sumDenominator;
}